@@ -8,25 +8,111 @@ use strict;
 our @www_whois = qw(
     VN
     AC
-    BZ
     TJ
+    CM
 );
-# Candidates for www_whois: DO, IO, MG, MN, SH, TM, TP, ZA
+# Candidates for www_whois: DO, IO, MG, SH, TM, TP, ZA
 
 our %servers = qw(
     RU          whois.ripn.net
     SU          whois.ripn.net
-    
-    COM.RU	whois.ripn.net
-    NET.RU	whois.ripn.net
-    ORG.RU	whois.ripn.net
-    PP.RU	whois.ripn.net
-    SPB.RU	whois.relcom.ru
-    MSK.RU	whois.relcom.ru
-    RU.NET	whois.relcom.ru
-    MSK.SU      whois.relcom.ru
-    INT.RU      whois.int.ru    
-    NNOV.RU     whois.nnov.ru
+    XN--P1AI	whois.ripn.net
+
+    COM.RU	whois.nic.ru
+    NET.RU	whois.nic.ru
+    ORG.RU	whois.nic.ru
+    PP.RU	whois.nic.ru
+    RU.NET	whois.nic.ru
+    INT.RU  whois.int.ru
+
+    ABKHAZIA.SU         whois.nic.ru
+    ADYGEYA.RU          whois.nic.ru
+    ADYGEYA.SU          whois.nic.ru
+    AKTYUBINSK.SU       whois.nic.ru
+    AMURSK.RU           whois.nic.ru
+    ARKHANGELSK.SU      whois.nic.ru
+    ARMENIA.SU          whois.nic.ru
+    ASHGABAD.SU         whois.nic.ru
+    AZERBAIJAN.SU       whois.nic.ru
+    BALASHOV.SU         whois.nic.ru
+    BASHKIRIA.RU        whois.nic.ru
+    BASHKIRIA.SU        whois.nic.ru
+    BELGOROD.RU         whois.nic.ru
+    BELGOROD.SU         whois.nic.ru
+    BIR.RU              whois.nic.ru
+    BRYANSK.SU          whois.nic.ru
+    BUKHARA.SU          whois.nic.ru
+    CBG.RU              whois.nic.ru
+    CHELYABINSK.RU      whois.nic.ru
+    CHIMKENT.SU         whois.nic.ru
+    CMW.RU              whois.nic.ru
+    DAGESTAN.RU         whois.nic.ru
+    DAGESTAN.SU         whois.nic.ru
+    DUDINKA.RU          whois.nic.ru
+    EAST-KAZAKHSTAN.SU  whois.nic.ru
+    EXNET.SU            whois.nic.ru
+    FAREAST.RU          whois.nic.ru
+    GEORGIA.SU          whois.nic.ru
+    GROZNY.RU           whois.nic.ru
+    GROZNY.SU           whois.nic.ru
+    IVANOVO.SU          whois.nic.ru
+    JAMBYL.SU           whois.nic.ru
+    JAR.RU              whois.nic.ru
+    JOSHKAR-OLA.RU      whois.nic.ru
+    KALMYKIA.RU         whois.nic.ru
+    KALMYKIA.SU         whois.nic.ru
+    KALUGA.SU           whois.nic.ru
+    KARACOL.SU          whois.nic.ru
+    KARAGANDA.SU        whois.nic.ru
+    KARELIA.SU          whois.nic.ru
+    KCHR.RU             whois.nic.ru
+    KHAKASSIA.SU        whois.nic.ru
+    KOMI.SU             whois.nic.ru
+    KRASNODAR.SU        whois.nic.ru
+    KURGAN.SU           whois.nic.ru
+    KUSTANAI.RU         whois.nic.ru
+    KUSTANAI.SU         whois.nic.ru
+    MANGYSHLAK.SU       whois.nic.ru
+    MARINE.RU           whois.nic.ru
+    MORDOVIA.RU         whois.nic.ru
+    MORDOVIA.SU         whois.nic.ru
+    MSK.RU              whois.nic.ru
+    MSK.SU              whois.nic.ru
+    MURMANSK.SU         whois.nic.ru
+    MYTIS.RU            whois.nic.ru
+    NALCHIK.RU          whois.nic.ru
+    NALCHIK.SU          whois.nic.ru
+    NAVOI.SU            whois.nic.ru
+    NNOV.RU             whois.nnov.ru
+    NORILSK.RU          whois.nic.ru
+    NORTH-KAZAKHSTAN.SU whois.nic.ru
+    NOV.RU              whois.nic.ru
+    NOV.SU              whois.nic.ru
+    OBNINSK.SU          whois.nic.ru
+    PALANA.RU           whois.nic.ru
+    PENZA.SU            whois.nic.ru
+    POKROVSK.SU         whois.nic.ru
+    PYATIGORSK.RU       whois.nic.ru
+    RU.NET              whois.nic.ru
+    SIMBIRSK.RU         whois.nic.ru
+    SOCHI.SU            whois.nic.ru
+    SPB.RU              whois.nic.ru
+    SPB.SU              whois.nic.ru
+    TASHKENT.SU         whois.nic.ru
+    TERMEZ.SU           whois.nic.ru
+    TOGLIATTI.SU        whois.nic.ru
+    TROITSK.SU          whois.nic.ru
+    TSARITSYN.RU        whois.nic.ru
+    TSELINOGRAD.SU      whois.nic.ru
+    TULA.SU             whois.nic.ru
+    TUVA.SU             whois.nic.ru
+    VLADIKAVKAZ.RU      whois.nic.ru
+    VLADIKAVKAZ.SU      whois.nic.ru
+    VLADIMIR.RU         whois.nic.ru
+    VLADIMIR.SU         whois.nic.ru
+    VOLOGDA.SU          whois.nic.ru
+    YAKUTIA.SU          whois.nic.ru
+    YEKATERINBURG.RU    whois.nic.ru
 
     NS     whois.nsiregistry.net
     RIPE   whois.ripe.net
@@ -53,14 +139,15 @@ our %servers = qw(
     ORG    whois.pir.org
     PRO    whois.registrypro.pro
     TEL    whois-tel.neustar.biz
-    TV     tvwhois.verisign-grs.com
-    WS     whois.worldsite.ws
-
-    NF     whois.nic.cx
+    TRAVEL whois.nic.travel
 
+    TV  tvwhois.verisign-grs.com
+    WS  whois.worldsite.ws
+    NF  whois.nic.cx
     AC  whois.nic.ac
     AG  whois.nic.ag
     AM  whois.amnic.net
+    AS  whois.nic.as
     AT  whois.nic.at
     AU  whois.aunic.net
     BE  whois.dns.be
@@ -72,6 +159,7 @@ our %servers = qw(
     CI  whois.nic.ci
     CL  Whois.nic.cl
     CN  whois.cnnic.net.cn
+    CO  whois.nic.co
     CX  whois.nic.cx
     CZ  whois.nic.cz
     DE  whois.denic.de
@@ -80,15 +168,21 @@ our %servers = qw(
     EE  whois.eenet.ee
     EU  whois.eu
     FI  whois.ficora.fi
+    FM  whois.nic.fm
+    FO  whois.ripe.net
     FR  whois.nic.fr
+    GD  whois.adamsnames.com
     GG  whois.channelisles.net
     GI  whois2.afilias-grs.net
     GS  whois.nic.gs
     HU  whois.nic.hu
     HK  whois.hkirc.hk
+    HM  whois.registry.hm
     HN  whois2.afilias-grs.net
+    HT  whois.nic.ht
     IE  whois.domainregistry.ie
     IL  whois.isoc.org.il
+    IM  whois.nic.im
     IN  whois.inregistry.net
     IO  whois.nic.io
     IS  whois.isnic.is
@@ -96,15 +190,20 @@ our %servers = qw(
     JE  whois.channelisles.net
     JP  whois.jprs.jp
     KE  whois.kenic.or.ke
+    KG  whois.domain.kg
+    KI  whois.nic.ki
     KR  whois.nic.or.kr
     KZ  whois.nic.kz
-    LA  whois2.afilias-grs.net
+    LA  whois.nic.la
+    LC  whois2.afilias-grs.net
     LI  whois.nic.li
     LT  whois.domreg.lt
     LU  whois.dns.lu
     LV  whois.nic.lv
+    MC  whois.ripe.net
+    MD  whois.nic.md
     MG  whois.nic.mg
-    MN  whois.nic.mn
+    MN  whois2.afilias-grs.net
     MS  whois.nic.ms
     MU  whois.nic.mu
     MX  whois.nic.mx
@@ -120,22 +219,26 @@ our %servers = qw(
     PT  whois.dns.pt
     RE  whois.nic.re
     RO  whois.rotld.ro
-    SB  whois.nic.net.sb
+    SB  whois.nic.sb
     SC  whois2.afilias-grs.net
     SE  whois.iis.se
     SG  whois.nic.net.sg
     SH  whois.nic.sh
     SI  whois.arnes.si
+    SK  whois.ripe.net
+    SM  whois.ripe.net
     ST  whois.nic.st
     TC  whois.adamsnames.tc
     TF  whois.nic.tf
+    TH  whois.nic.uk
     TK  whois.dot.tk
     TL  whois.nic.tl
     TM  whois.nic.tm
     TO  whois.tonic.to
     TR  whois.nic.tr
     TW  whois.twnic.net.tw
-    UA  whois.net.ua
+    UA  whois.com.ua
+    NET.UA  whois.net.ua
     UK  whois.nic.uk
     US  whois.nic.us
     UZ  whois.cctld.uz
@@ -145,18 +248,163 @@ our %servers = qw(
     WF  whois.nic.wf
     YT  whois.nic.yt
 
+    ASN.AU        whois.aunic.net
+    COM.AU        whois.aunic.net
+    CONF.AU       whois.aunic.net
+    CSIRO.AU      whois.aunic.net
+    EDU.AU        whois.aunic.net
+    GOV.AU        whois.aunic.net
+    ID.AU         whois.aunic.net
+    INFO.AU       whois.aunic.net
+    NET.AU        whois.aunic.net
+    ORG.AU        whois.aunic.net
+    EMU.ID.AU     whois.aunic.net
+    WATTLE.ID.AU  whois.aunic.net
+
+    ADM.BR  whois.nic.br
+    ADV.BR  whois.nic.br
+    AGR.BR  whois.nic.br
+    AM.BR   whois.nic.br
+    ARQ.BR  whois.nic.br
+    ART.BR  whois.nic.br
+    ATO.BR  whois.nic.br
+    BIO.BR  whois.nic.br
+    BMD.BR  whois.nic.br
+    CIM.BR  whois.nic.br
+    CNG.BR  whois.nic.br
+    CNT.BR  whois.nic.br
+    COM.BR  whois.nic.br
+    ECN.BR  whois.nic.br
+    EDU.BR  whois.nic.br
+    ENG.BR  whois.nic.br
+    ESP.BR  whois.nic.br
+    ETC.BR  whois.nic.br
+    ETI.BR  whois.nic.br
+    FAR.BR  whois.nic.br
+    FM.BR   whois.nic.br
+    FND.BR  whois.nic.br
+    FOT.BR  whois.nic.br
+    FST.BR  whois.nic.br
+    G12.BR  whois.nic.br
+    GGF.BR  whois.nic.br
+    GOV.BR  whois.nic.br
+    IMB.BR  whois.nic.br
+    IND.BR  whois.nic.br
+    INF.BR  whois.nic.br
+    JOR.BR  whois.nic.br
+    LEL.BR  whois.nic.br
+    MAT.BR  whois.nic.br
+    MED.BR  whois.nic.br
+    MIL.BR  whois.nic.br
+    MUS.BR  whois.nic.br
+    NET.BR  whois.nic.br
+    NOM.BR  whois.nic.br
+    NOT.BR  whois.nic.br
+    NTR.BR  whois.nic.br
+    ODO.BR  whois.nic.br
+    OOP.BR  whois.nic.br
+    ORG.BR  whois.nic.br
+    PPG.BR  whois.nic.br
+    PRO.BR  whois.nic.br
+    PSC.BR  whois.nic.br
+    PSI.BR  whois.nic.br
+    QSL.BR  whois.nic.br
+    REC.BR  whois.nic.br
+    SLG.BR  whois.nic.br
+    SRV.BR  whois.nic.br
+    TMP.BR  whois.nic.br
+    TRD.BR  whois.nic.br
+    TUR.BR  whois.nic.br
+    TV.BR   whois.nic.br
+    VET.BR  whois.nic.br
+    ZLG.BR  whois.nic.br
+
+    AC.CN  whois.cnnic.net.cn
+    AH.CN  whois.cnnic.net.cn
+    BJ.CN  whois.cnnic.net.cn
+    COM.CN whois.cnnic.net.cn
+    CQ.CN  whois.cnnic.net.cn
+    FJ.CN  whois.cnnic.net.cn
+    GD.CN  whois.cnnic.net.cn
+    GOV.CN whois.cnnic.net.cn
+    GS.CN  whois.cnnic.net.cn
+    GX.CN  whois.cnnic.net.cn
+    GZ.CN  whois.cnnic.net.cn
+    HA.CN  whois.cnnic.net.cn
+    HB.CN  whois.cnnic.net.cn
+    HE.CN  whois.cnnic.net.cn
+    HI.CN  whois.cnnic.net.cn
+    HK.CN  whois.cnnic.net.cn
+    HL.CN  whois.cnnic.net.cn
+    HN.CN  whois.cnnic.net.cn
+    JL.CN  whois.cnnic.net.cn
+    JS.CN  whois.cnnic.net.cn
+    JX.CN  whois.cnnic.net.cn
+    LN.CN  whois.cnnic.net.cn
+    MO.CN  whois.cnnic.net.cn
+    NET.CN whois.cnnic.net.cn
+    NM.CN  whois.cnnic.net.cn
+    NX.CN  whois.cnnic.net.cn
+    ORG.CN whois.cnnic.net.cn
+    QH.CN  whois.cnnic.net.cn
+    SC.CN  whois.cnnic.net.cn
+    SD.CN  whois.cnnic.net.cn
+    SH.CN  whois.cnnic.net.cn
+    SN.CN  whois.cnnic.net.cn
+    SX.CN  whois.cnnic.net.cn
+    TJ.CN  whois.cnnic.net.cn
+    TW.CN  whois.cnnic.net.cn
+    XJ.CN  whois.cnnic.net.cn
+    XZ.CN  whois.cnnic.net.cn
+    YN.CN  whois.cnnic.net.cn
+    ZJ.CN  whois.cnnic.net.cn
+
+    COM.HK  whois.hknic.net.hk
+    GOV.HK  whois.hknic.net.hk
+    NET.HK  whois.hknic.net.hk
+    ORG.HK  whois.hknic.net.hk
+
+    AC.JP  whois.jprs.jp
+    AD.JP  whois.jprs.jp
+    CO.JP  whois.jprs.jp
+    GR.JP  whois.jprs.jp
+    NE.JP  whois.jprs.jp
+    OR.JP  whois.jprs.jp
+
+    COM.MX  whois.nic.mx
+    GOB.MX  whois.nic.mx
+    NET.MX  whois.nic.mx
+
+    COM.TW  whois.twnic.net
+    IDV.TW  whois.twnic.net
+    NET.TW  whois.twnic.net
+    ORG.TW  whois.twnic.net
+
     NET.UA      delta.hostmaster.net.ua
+    COM.UA      whois.com.ua
+    ORG.UA      whois.com.ua
+    BIZ.UA      whois.biz.ua
+    CO.UA       whois.co.ua
+    PP.UA	whois.pp.ua
+    KIEV.UA     whois.com.ua
     DN.UA       whois.dn.ua
     LG.UA       whois.lg.ua
     OD.UA       whois.od.ua
-    AC.UK	whois.ja.net
-    GOV.UK	whois.ja.net
+    IN.UA       whois.in.ua
 
-    XN---P1AG	ru.whois.i-dns.net
-    XN---P1AG	ru.whois.i-dns.net
-    XN---J1AEF	whois.i-dns.net
-    XN---E1APQ	whois.i-dns.net
-    XN---C1AVG	whois.i-dns.net
+    AC.UK	whois.ja.net
+    CO.UK   whois.nic.uk
+    GOV.UK	whois.ja.net
+    LTD.UK  whois.nic.uk
+    NET.UK  whois.nic.uk
+    ORG.UK  whois.nic.uk
+    PLC.UK  whois.nic.uk
+
+    XN--P1AG	ru.whois.i-dns.net
+    XN--P1AG	ru.whois.i-dns.net
+    XN--J1AEF	whois.i-dns.net
+    XN--E1APQ	whois.i-dns.net
+    XN--C1AVG	whois.i-dns.net
 
     EU.COM      whois.centralnic.com
     GB.COM      whois.centralnic.com
@@ -181,6 +429,13 @@ our %servers = qw(
     SE.NET	whois.centralnic.com
 
     AE.ORG	whois.centralnic.com
+
+    ORG.NS	whois.pir.org
+    BIZ.NS	whois.biz
+    NAME.NS	whois.nic.name
+    SO          whois.nic.so
+    BZ          whois.belizenic.bz
+    XXX         whois.nic.xxx
 );
 
 
@@ -209,11 +464,16 @@ our %codepages = (
     'whois.nic.hu'       => 'iso-8859-1',
     'whois.dns.pt'       => 'iso-8859-1',
     'whois.net.ua'       => 'koi8-u',
+    'whois.com.ua'       => 'koi8-u',
+    'whois.biz.ua'       => 'koi8-u',
+    'whois.co.ua'        => 'koi8-u',
     'whois.dn.ua'        => 'koi8-u',
     'whois.lg.ua'        => 'koi8-u',
     'whois.od.ua'        => 'koi8-u',
+    'whois.in.ua'        => 'koi8-u',
     'whois.jprs.jp'      => 'iso-2022-jp',
     'whois.nic.or.kr'    => 'euc-kr',
+    'whois.domain.kg'    => 'cp-1251',
 );
 
 
@@ -222,11 +482,22 @@ our %notfound = (
     'whois.ripe.net'        => 'No entries found',
 
     'whois.ripn.net'        => 'No entries found',
-    'whois.relcom.ru'       => 'No entries found',
+    'whois.nic.ru'       => 'No entries found',
     'whois.nnov.ru'         => 'No entries found',
     'whois.int.ru'          => 'No entries found',
     'whois.reg.ru'          => '^Domain \S+ not found',
-    
+
+    'whois.com.ua'          => 'No entries found for',
+    'whois.co.ua'           => 'No entries found',
+    'whois.biz.ua'          => 'No entries found',
+    'whois.net.ua'            => 'No entries found for domain',
+    'delta.hostmaster.net.ua' => 'No entries found for domain',
+    'whois.pp.ua'             => 'No entries found',
+    'whois.dn.ua'             => 'No match record found',
+    'whois.lg.ua'             => 'No match record found',
+    'whois.od.ua'             => 'No match record found',
+    'whois.in.ua'             => 'No records found',
+
     'whois.aero'                 => '^NOT FOUND',
     'whois.nic.asia'             => '^NOT FOUND',
     'whois.biz'                  => '^Not found:',
@@ -246,47 +517,58 @@ our %notfound = (
     'ccwhois.verisign-grs.com'   => '^No match for',
     'jobswhois.verisign-grs.com' => '^No match for',
     'tvwhois.verisign-grs.com'   => '^No match for',
-    'whois.registrypro.pro'      => '^Unknown domain',
+    'whois.registrypro.pro'      => '^No match',
     'whois.worldsite.ws'         => 'No match for',
+    'whois.nic.travel'           => 'Not found: \S+',
 
     'whois.nic.ag'            => 'NOT FOUND',
+    'whois.nic.as'            => 'Domain Not Found',
     'whois.nic.at'            => 'nothing found',
+    'whois.nic.br'            => '^% No match for domain',
     'whois.amnic.net'         => 'No match',
     'whois.aunic.net'         => 'No Data Found',
     'whois.dns.be'            => '^Status:\s+FREE',
     'whois.register.bg'       => '^Domain name \S+ does not exist',
     'whois.registro.br'       => 'No match for',
+    'whois.registry.hm'       => 'Domain not found',
+    'whois.nic.ht'            => 'Status: Not Registered',
     'whois.cira.ca'           => '^Domain status\:\s+AVAIL',
     'whois.nic.ch'            => '^We do not have an entry in our database matching your',
     'whois.nic.ci'            => '^Domain \S+ not found',
     'whois.nic.cl'            => '\: no existe',
     'whois.nic.cx'            => 'Status\: Not Registered',
     'whois.nic.cz'            => 'No data found',
-    'whois.denic.de'          => 'No entries found',
+    'whois.denic.de'          => 'Status\: free',
     'whois.nic.dm'            => 'Status\: Not Registered',
     'whois.dk-hostmaster.dk'  => '^No entries found for',
     'whois.eenet.ee'          => '^NOT FOUND',
     'whois.eu'                => '^Status:\s+AVAILABLE',
     'whois.ficora.fi'         => 'Domain not found',
+    'whois.nic.fm'            => 'Status: Not Registered',
     'whois.nic.fr'            => 'No entries found',
     'whois.channelisles.net'  => 'No information found',
     'whois.nic.gs'            => 'Status: Not Registered',
     'whois.hkirc.hk'          => '^Domain Not Found',
+    'whois.hknic.net.hk'      => 'Domain Not Found',
     'whois.nic.hu'            => 'No match',
-    'whois.domainregistry.ie' => 'There was no match',    
+    'whois.domainregistry.ie' => 'There was no match',
     'whois.isoc.org.il'       => 'No data was found',
+    'whois.nic.im'            => 'The domain \S+ was not found',
     'whois.inregistry.net'    => 'NOT FOUND',
     'whois.nic.io'            => 'Domain \S+ - Available',
     'whois.isnic.is'          => 'No entries found',
     'whois.nic.it'            => 'Status:\s+AVAILABLE',
     'whois.jprs.jp'           => 'No match',
     'whois.kenic.or.ke'       => 'No match found',
+    'whois.nic.ki'            => 'Status: Not Registered',
     'whois.nic.or.kr'         => 'Above domain name is not registered',
     'whois.nic.kz'            => 'Nothing found for this query',
+    'whois.nic.la'            => 'DOMAIN NOT FOUND',
     'whois.nic.li'            => 'We do not have an entry',
     'whois.domreg.lt'         => '^Status:\s+available',
     'whois.dns.lu'            => 'No such domain',
     'whois.nic.lv'            => 'Nothing found',
+    'whois.nic.md'            => 'No match for',
     'whois.nic.ms'            => 'Status\: Not Registered',
     'whois.nic.mx'            => 'Object_Not_Found',
     'whois.mynic.net.my'      => '^Domain Name \S+ does not',
@@ -301,31 +583,29 @@ our %notfound = (
     'whois.dns.pt'            => 'no match',
     'whois.nic.re'            => 'No entries found',
     'whois.rotld.ro'          => 'No entries found',
+    'whois.nic.sb'            => 'Status: Not Registered',
     'whois.iis.se'            => '^\S+ not found',
     'whois.nic.net.sg'        => '^Domain Not Found',
     'whois.nic.sh'            => '^Domain \S+ - Available',
     'whois.arnes.si'          => 'No entries found',
     'whois.nic.st'            => '^No entries found',
     'whois.adamsnames.tc'     => '^\S+ is not registered',
+    'whois.adamsnames.com'    => '^\S+ is not registered',
     'whois.nic.tl'            => 'Status\: Not Registered',
     'whois.nic.tf'            => 'No entries found',
     'whois.dot.tk'            => 'domain name not known',
     'whois.nic.tm'            => '^Domain \S+ - Available',
     'whois.tonic.to'          => 'No match for',
-    'whois.twnic.net.tw'      => '^No Found',    
-    'whois.net.ua'            => 'No entries found for domain',
-    'delta.hostmaster.net.ua' => 'No entries found for domain',
-    'whois.dn.ua'             => 'No match record found',
-    'whois.lg.ua'             => 'No match record found',
-    'whois.od.ua'             => 'No match record found',
+    'whois.twnic.net'         => 'No Found',
+    'whois.twnic.net.tw'      => '^No Found',
     'whois.nic.uk'            => 'No match for',
     'whois.ja.net'            => '^No such domain',
     'whois.nic.us'            => '^Not found',
     'whois.nic.ve'            => 'No match for',
     'whois.nic.wf'            => 'No entries found',
-    'whois.nic.yt'            => 'No entries found', 
+    'whois.nic.yt'            => 'No entries found',
+
 
-    
     'whois.nsiregistry.net'     => 'No match for',
 
     'whois.007names.com'        => '^The Domain Name \S+ does not exist',
@@ -378,8 +658,15 @@ our %notfound = (
     'whois.worldnames.net'      => 'NO MATCH for domain',
      # 'whois.yournamemonkey.com' -- need try again
     'whois.cnnic.net.cn'	=> 'no matching record',
-    
-    'www_whois'                 => '(Available|no records found|is free)', # for VN | TJ | TJ zone
+    'whois.nic.co'              => 'Not found: \S+',
+    'me.whois-servers.net'	=> 'NOT FOUND',
+    'whois.domain.kg'           => 'Data not found. This domain is available for registration.',
+    'whois.nic.so'              => 'Not\s+found:',
+
+    # for VN | TJ | CM zones
+    'www_whois'                 => '(Available|no records found|is free|Not Registered)',
+
+    'whois.nic.xxx'             => 'NOT FOUND',
 );
 
 our %strip = (
@@ -393,7 +680,7 @@ our %strip = (
 	'^%',
     ],
 
-    
+
     'whois.ripn.net' => [
 	'^%',
 	'Last updated on ',
@@ -468,6 +755,9 @@ our %strip = (
 	'^whatsoever, you agree to abide by',
 	'^NOTE\: FAILURE TO LOCATE A RECORD',
 	'^OF THE AVAILABILITY OF A DOMAIN NAME',
+	'^NeuStar, Inc\., the Registry',
+	'^NeuStar registry database',
+	'^allowed without NeuStar',
     ],
     'whois-tel.neustar.biz' => [ # .tel
         '^>>>> Whois database was last updated',
@@ -923,7 +1213,7 @@ our %strip = (
 	'^in the .* to be registered',
 	'^prohibited.',
     ],
-    
+
 
 
     'whois.nic.ag' => [
@@ -980,20 +1270,8 @@ our %strip = (
     ],
     'whois.nic.cx' => [
 	'^TERMS OF USE\: You are not',
-	'^database through the use of',
-	'^automated\.  Whois database',
-	'^community on behalf of CoCCA',
-	'^The data is for information',
-	'^guarantee its accuracy',
-	'^by the following terms of use',
-	'^for lawful purposes and that',
-	'^to\: \(1\) allow, enable, or',
-	'^unsolicited, commercial',
-	'^or facsimile; or \(2\) enable',
-	'^that apply to CoCCA it',
-	'^compilation, repackaging, dissemination',
-	'^expressly prohibited',
-	'^CoCCA Helpdesk',
+        '^CiiA makes every effort to maintain the completeness',
+        'CiiA, All rights reserved',
 	'^Domain Information$',
     ],
     'whois.nic.cz' => [
@@ -1063,7 +1341,7 @@ our %strip = (
 	'^The Registry contains ONLY',
 	'^.* and .*\\.HK domains.',
     ],
-    'whois2.afilias-grs.net' => [ # .GI .HN .LA .SC .VC
+    'whois2.afilias-grs.net' => [ # .GI .HN .LC .SC .VC
 	'^Access to CCTLD WHOIS',
 	'^determining the contents',
 	'^Afilias registry database',
@@ -1204,20 +1482,8 @@ our %strip = (
     ],
     'whois.nic.mu' => [
 	'^TERMS OF USE\: You are not',
-	'^database through the use',
-	'^automated\.  Whois',
-	'^community on behalf of',
-	'^The data is for information',
-	'^guarantee its accuracy',
-	'^by the following terms',
-	'^for lawful purposes and',
-	'^to\: \(1\) allow, enable',
-	'^unsolicited, commercial',
-	'^or facsimile; or \(2',
-	'^that apply to CoCCA it',
-	'^compilation, repackaging',
-	'^expressly prohibited',
-	'^CoCCA Helpdesk',
+        '^Internet Direct Ltd makes every effort to maintain the completeness',
+        'Internet Direct Ltd, All rights reserved',
 	'^Domain Information$',
     ],
     'whois.nic.mx' => [
@@ -1305,6 +1571,17 @@ our %strip = (
 	'of any such activities or',
 	'Copyright \(c\) The',
 	'Netherlands \(SIDN\)',
+
+        '^These restrictions apply equally',
+        '^reproductions and publications',
+        '^reasonable, necessary and solely',
+        '^activities referred to in the',
+        '^Registrars',
+        '^action. Anyone who is aware',
+        '^in the Netherlands',
+        '^\(SIDN\) Dutch Copyright Act',
+        '^subsection 1, clause 1\).',
+
     ],
     'whois.norid.no' => [
 	'^%',
@@ -1432,6 +1709,13 @@ our %strip = (
     'whois.od.ua' => [
         '^%',
     ],
+    'whois.com.ua' => [
+	'^% This is the Ukrainian',
+	'^% Rights restricted',
+        '^%$',
+        '^% % .UA whois',
+        '^% =====',
+    ],
     'whois.nic.uk' => [
 	'^This WHOIS information is',
 	'^for \.uk domain names',
@@ -1665,17 +1949,182 @@ our %strip = (
 	'^reserves the right to modify',
 	'^The Registry database contains',
 	'^Registrars\.$',
+	'^Whois Server Version',
+    ],
+    'whois.nic.travel' => [
+        '^>>>> Whois database was last updated',
+        '^Tralliance, Inc., the Registry Operator',
+        '^for the WHOIS database through',
+        '^is provided to you for',
+        '^persons in determining',
+        '^Tralliance registry database',
+        '^"as is" and does not',
+        '^agree that you will',
+        '^circumstances will',
+        '^support the transmission',
+        '^solicitations via direct mail',
+        '^contravention of any applicable',
+        '^enable high volume, automated',
+        '^\(or its systems\).  Compilation',
+        '^WHOIS database in its entirety',
+        '^allowed without Tralliance',
+        '^right to modify or change',
+        '^subsequent notification of any',
+        '^whatsoever, you agree to abide',
+        '^NOTE: FAILURE TO LOCATE A RECORD',
+        '^OF THE AVAILABILITY OF A DOMAIN NAME',
+    ],
+    'whois.nic.ht' => [
+        '^TERMS OF USE: You are not authorized',
+        '^database through the use of electronic',
+        '^automated.  Whois database is provided',
+        '^community on by of Consortium FDS/RDDH',
+        '^The data is for information purposes only.',
+        '^guarantee its accuracy. By submitting a',
+        '^by the following terms of use:',
+        '^for lawful purposes and that under',
+        '^to: \(1\) allow, enable, or',
+        '^unsolicited, commercial',
+        '^or facsimile; or \(2\) enable',
+        '^that apply to Consortium FDS/RDDH',
+        '^compilation, repackaging',
+        '^expressly prohibited.',
+        '^Domain Information$',
+    ],
+    'whois.nic.ki' => [
+        '^TERMS OF USE: You are not',
+        '^database through the',
+        '^automated.  Whois database',
+        '^community on behalf of CoCCA',
+        '^The data is for information purposes',
+        '^guarantee its accuracy. By',
+        '^by the following terms of use',
+        '^for lawful purposes and that',
+        '^to: \(1\) allow, enable, or',
+        '^unsolicited, commercial',
+        '^or facsimile; or \(2\) enable',
+        '^that apply to CoCCA it\'s members',
+        '^compilation, repackaging',
+        '^expressly prohibited.',
+    ],
+    'whois.nic.la' => [
+        '^This whois service is provided by',
+        '^pertaining to Internet domain names',
+        '^using this service you are agreeing',
+        '^here for any purpose other than',
+        '^to store or reproduce this data',
+    ],
+    'whois.nic.sb' => [
+        '^TERMS OF USE: You are not authorized',
+        '^The data is for information purposes only',
+        '^CoCCA Helpdesk \| http://helpdesk.cocca.cx',
+        '^Domain Information$',
+    ],
+    'whois.nic.tl' => [
+        '^TERMS OF USE: You are not authorized',
+        '^database through the use of',
+        '^automated.  Whois database is',
+        '^community on behalf of CoCCA',
+        '^The data is for information',
+        '^guarantee its accuracy. By',
+        '^by the following terms of use',
+        '^for lawful purposes and that',
+        '^to: \(1\) allow, enable, or',
+        '^unsolicited, commercial',
+        '^or facsimile; or',
+        '^that apply to CoCCA',
+        '^compilation, repackaging',
+        '^expressly prohibited',
+        '^CoCCA Helpdesk',
+        '^Domain Information$',
+    ],
+    'whois.nic.fm' => [
+        '^TERMS OF USE',
+        '^dotFM makes every effort',
+        'is a registered trademark of BRS Media Inc.',
+        '^Domain Information$',
+    ],
+    'whois.nic.co' => [
+        '^>>>> Whois database was last',
+        '^.CO Internet, S.A.S., the',
+        '^information for the WHOIS',
+        '^This information is provided',
+        '^designed to assist persons',
+        '^registration record in the',
+        '^information available to you',
+        '^By submitting a WHOIS query',
+        '^lawful purposes and that',
+        '^\(1\) to allow, enable',
+        '^unsolicited, commercial',
+        '^electronic mail, or by',
+        '^data and privacy protection',
+        '^electronic processes that',
+        '^repackaging, dissemination',
+        '^entirety, or of a substantial',
+        '^.CO Internet',
+        '^change these conditions at',
+        '^of any kind. By executing',
+        '^abide by these terms.',
+        '^NOTE: FAILURE TO LOCATE',
+        '^OF THE AVAILABILITY',
+        '^All domain names are subject',
+        '^rules.  For details, please',
+    ],
+    'whois.domain.kg' => [
+        '^%',
+    ],
+    'whois.belizenic.bz' => [
+        '^The data in BelizeNIC registrar WHOIS database is provided to you by',
+        '^BelizeNIC registrar for information purposes only, that is, to assist you in',
+        '^obtaining information about or related to a domain name registration',
+        '^record. BelizeNIC registrar makes this information available "as is,"',
+        '^and',
+        '^does not guarantee its accuracy. By submitting a WHOIS query, you',
+        '^agree that you will use this data only for lawful purposes and that,',
+        '^under no circumstances',
+        '^or otherwise support the transmission of mass unsolicited, commercial',
+        '^advertising or solicitations via direct mail, electronic mail, or by',
+        '^telephone; ',
+        '^that apply to BelizeNIC',
+        '^repackaging, dissemination or other use of this data is expressly',
+        '^prohibited without the prior written consent of BelizeNIC registrar.',
+        '^BelizeNIC registrar reserves the right to modify these terms at any time.',
+        '^By submitting this query, you agree to abide by these terms.',
+    ],
+    'whois.nic.xxx' => [
+        '^Access to the .XXX WHOIS information is provided to assist persons in',
+        '^determining the contents of a domain name registration record in the',
+        '^ICM Registry database. The data in this record is provided by',
+        '^ICM Registry for informational purposes only, and ICM does not',
+        '^guarantee its accuracy. This service is intended only for query-based',
+        '^access. You agree that you will use this data only for lawful purposes',
+        '^and that, under no circumstances will you use this data to',
+        '^enable, or otherwise support the transmission by e-mail, telephone, or',
+        '^facsimile of mass unsolicited, commercial advertising or solicitations',
+        '^to entities other than the data recipient',
+        '^\(b\) enable high volume, automated, electronic processes that send',
+        '^queries or data to the systems of Registry Operator, a Registrar, or',
+        '^ICM except as reasonably necessary to register domain names or',
+        '^modify existing registrations. All rights reserved. ICM reserves',
+        '^the right to modify these terms at any time. By submitting this query,',
+        '^you agree to abide by this policy.',
     ],
 );
 
 our %exceed = (
-    'whois.eu' => 'Excessive querying, grace period of',
+    'whois.eu' => '(?:Excessive querying, grace period of|Still in grace period, wait)',
     'whois.dns.lu' => 'Excessive querying, grace period of',
     'whois.mynic.net.my' => 'Query limitation is',
     'whois.ripn.net' => 'exceeded allowed connection rate',
     'whois.domain-registry.nl' => 'too many requests',
     'whois.nic.uk' => 'and will be replenished',
     'whois.networksolutions.com' => 'contained within a list of IP addresses that may have failed',
+    'whois.worldsite.ws' => 'You exceeded the maximum',
+    'whois.tucows.com'  => '(?:Maximum Daily connection limit reached|exceeded maximum connection limit)',
+    'whois.centralnic.com'  => 'Query rate of \\d+',
+    'whois.pir.org'  => 'WHOIS LIMIT EXCEEDED',
+    'whois.nic.ms'   => 'Look up quota exceeded',
+    'whois.nic.gs'   => 'look up quota exceeded',
 );
 
 our $default_ban_time = 60;
@@ -1683,4 +2132,26 @@ our %ban_time = (
     'whois.ripn.net'  => 60,
 );
 
+# Whois servers which has no idn support
+our %whois_servers_with_no_idn_support = (
+    'whois.melbourneit.com'  => 1,
+);
+
+# Internal postprocessing subroutines
+our %postprocess = (
+    'whois.pp.ua'   => sub { $_[0] =~ s/[\x00\x0A]+$//; $_[0]; },
+);
+
+# Special query prefix strings for some servers
+our %query_prefix = (
+    'whois.crsnic.net'         => 'domain ',
+    'whois.denic.de'           => '-T dn,ace -C ISO-8859-1 ',
+    'whois.nic.name'           => 'domain=',
+
+    'whois.nic.name.ns'        => 'nameserver=',
+    'whois.pir.org.ns'         => 'HO ',
+    'whois.biz.ns'             => 'nameserver ',
+    'whois.nsiregistry.net.ns' => 'nameserver = ',
+);
+
 1;
